# Copyright (c) 2012 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
"""
This program takes an aut and smv file from a generated TuLiP controller
and automatically writes the MATLAB compatible script for that controller.

Run this program by typing "python programfile.py nameofautfile
nameofmatlabfile", aut and smv file shall have the same name.
Do not include file extensions.

Written by Robert Rogersten during SURF June 2012,
Co-mentors Mumu Xu, Necmiye Ozay and Ufuk Topcu.
"""

from __future__ import print_function
import re, copy, os, sys
import Queue


class AutomatonState:
    """AutomatonState class for representing a state in a finite state
    automaton.  An AutomatonState object contains the following
    fields:

    - `stateid`: an integer specifying the state id of this AutomatonState object.
    - `state`: a dictionary whose keys are the names of the variables
      and whose values are the values of the variables.
    - `transition`: a list of id's of the AutomatonState objects to
      which this AutomatonState object can transition.
    """
    def __init__(self, stateid=-1, state={},transition=[]):
        self.stateid = stateid
        self.state = copy.copy(state)
        self.transition = transition[:]

def question(string):
    """This function asks a yes/no question and returns the answer.

    @param string: The question to use as the prompt.

    @return: The "answer" return value is one of "yes" or "no".  The
        default is "yes". (The default occurs if the user only presses
        the RETURN button.)
    """
    default="yes"
    valid = {"yes":True,   "y":True,  "ye":True,
             "no":False,     "n":False}
    prompt = " [Y/n] "
    while True:
        print(string)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            print("Please respond with 'yes' or 'no'\n")

def load_file(aut_file):
    """Construct an AutomatonState object from aut_file and place in a Queue.

    @param aut_file: the name of the text file containing the
        automaton, or an (open) file-like object.
    """
    if isinstance(aut_file, str):
        f = open(aut_file, 'r')
    else:
        f = aut_file
    stateid = -1
    for line in f:
        # parse states
        if (line.find('State ') >= 0):
            stateid = re.search('State (\d+)', line)
            stateid = int(stateid.group(1))
            state = dict(re.findall('(\w+):(\w+)', line))
            state1 = dict(re.findall('(\w+):(-\w+)', line))
            state.update(state1)
        if re.search('successors', line):
            transition = list(re.findall('\d+', line))
            automaton=(stateid,state,transition)
            queue.put(automaton)
            queue1.put(automaton)
            queue2.put(automaton)

def read_variables(smv_file):
    """Put the enviroment and system variables from smv_file in two different
    Queues called system and enviroment.

    @param smv_file: the name of the text file containing the
        automaton, or an (open) file-like object.
    """
    if isinstance(smv_file, str):
        f = open(smv_file, 'r')
    else:
        f = smv_file
    for line in f:
        if re.search('MODULE env',line):
            for line in f:
                if re.search(' : ', line):
                    env = str(re.findall('(\w+) :', line))
                    env = env[2:len(env)-2]
                    enviroment.put(env)
                if re.search('MODULE sys',line):
                    break
        if re.search('MODULE sys',line):
            for line in f:
                if re.search(' : ', line):
                    sys = str(re.findall('(\w+) :', line))
                    sys = sys[2:len(sys)-2]
                    system.put(sys)

def write_startline(enviroment,system,f):
    """Write the first lines before the switch cases in the matlab file.

    Input:

    - enviroment queue
    - system queue
    - fileobject f
    """
    f.write('function [')
    for i in range(system.qsize()):
        count = system.qsize()
        temp = system.get()
        if count == i+1:
            f.write(temp)
        else:
            f.write(temp+',')
        system.put(temp)
    f.write('] = '+sys.argv[2]+'(')
    for i in range(enviroment.qsize()):
        count = enviroment.qsize()
        temp = enviroment.get()
        if count == i+1:
            f.write(temp)
        else:
            f.write(temp+',')
        enviroment.put(temp)
    f.write(")\nglobal state;\ncoder.extrinsic('disp');\nswitch state\n")

def write_case(enviroment,system,f,verbosem):
    """Write the switch cases in the matlab file.

    Input:

    - enviroment queue
    - system queue
    - fileobject f
    - verbosem
    """

    #for each case
    for i in range(queue.qsize()):
        f.write('\tcase '+str(i)+'\n')

        #for each condition within each case
        temp=queue.get()
        ef=0
        for k in range(queue1.qsize()):
            temp2=queue1.get()
            if str(k) in temp[2]:
                if ef == 0:
                    f.write('\t\tif ')
                    ef=1
                else:
                    f.write('\t\telseif ')
                for l in range(enviroment.qsize()):
                    count=enviroment.qsize()
                    temp1=enviroment.get()
                    if count == l+1:
                        f.write(temp1+' == '+temp2[1][temp1])
                    else:
                        f.write(temp1+' == '+temp2[1][temp1]+' && ')
                    enviroment.put(temp1)
                f.write('\n')
                if verbosem==1:
                    f.write('\t\t\tstate = '+str(temp2[0])+';\n')
                elif verbosem==0:
                    f.write('\t\t\tstate = '+str(temp2[0])+'\n')
                else:
                    raise Exception
                for l in range(system.qsize()):
                    temp1=system.get()
                    if verbosem==1:
                        f.write('\t\t\t'+temp1+' = '+temp2[1][temp1]+';\n')
                    elif verbosem==0:
                        f.write('\t\t\t'+temp1+' = '+temp2[1][temp1]+'\n')
                    else:
                        raise Exception
                    system.put(temp1)
            queue1.put(temp2)


        #else statement for each case
        if not temp[2]:
            for l in range(system.qsize()):
                temp1=system.get()
                if verbosem==1:
                    f.write('\t\t'+temp1+' = '+temp[1][temp1]+';\n')
                elif verbosem==0:
                    f.write('\t\t'+temp1+' = '+temp[1][temp1]+'\n')
                else:
                    raise Exception
                system.put(temp1)
        else:
            f.write('\t\telse\n')
            f.write("\t\t\tdisp('Cannot find a valid successor, environment assumption is like to be violated')\n")
            for l in range(system.qsize()):
                temp1=system.get()
                if verbosem==1:
                    f.write('\t\t\t'+temp1+' = '+temp[1][temp1]+';\n')
                elif verbosem==0:
                    f.write('\t\t\t'+temp1+' = '+temp[1][temp1]+'\n')
                else:
                    raise Exception
                system.put(temp1)
            f.write('\t\tend\n')
        queue.put(temp)

    #the last case is an otherwise statement
    f.write('\totherwise\n')
    f.write("\t\tdisp('Cannot find a valid successor, environment assumption is like to be violated')\n")
    for l in range(system.qsize()):
        temp1=system.get()
        if verbosem==1:
            f.write('\t\t'+temp1+' = 0;\n')
        elif verbosem==0:
            f.write('\t\t'+temp1+' = 0\n')
        else:
            raise Exception
        system.put(temp1)
    f.write('end')

def write_case_no(enviroment,system,f,verbosem):
    """Write the switch cases in the matlab file and exclude no
    successors.

    Input:

    - enviroment queue
    - system queue
    - fileobject f
    - verbosem
    """

    #for each case
    li=list()
    for i in range(queue.qsize()):
        q=queue.get()
        li.append(q[0])
        queue.put(q)

    for i in range(queue.qsize()):

        #for each condition within each case
        temp=queue.get()
        f.write('\tcase '+str(temp[0])+'\n')
        ef=0
        for k in range(queue2.qsize()):
            temp2=queue2.get()
            if str(k) in temp[2] and k in li:
                if ef == 0:
                    f.write('\t\tif ')
                    ef=1
                else:
                    f.write('\t\telseif ')
                for l in range(enviroment.qsize()):
                    count=enviroment.qsize()
                    temp1=enviroment.get()
                    if count == l+1:
                        f.write(temp1+' == '+temp2[1][temp1])
                    else:
                        f.write(temp1+' == '+temp2[1][temp1]+' && ')
                    enviroment.put(temp1)
                f.write('\n')
                if verbosem==1:
                    f.write('\t\t\tstate = '+str(k)+';\n')
                elif verbosem==0:
                    f.write('\t\t\tstate = '+str(k)+'\n')
                else:
                    raise Exception
                for l in range(system.qsize()):
                    temp1=system.get()
                    if verbosem==1:
                        f.write('\t\t\t'+temp1+' = '+temp2[1][temp1]+';\n')
                    elif verbosem==0:
                        f.write('\t\t\t'+temp1+' = '+temp2[1][temp1]+'\n')
                    else:
                        raise Exception
                    system.put(temp1)
            queue2.put(temp2)


        #else statement for each case
        if not temp[2]:
            for l in range(system.qsize()):
                temp1=system.get()
                if verbosem==1:
                    f.write('\t\t'+temp1+' = '+temp[1][temp1]+';\n')
                elif verbosem==0:
                    f.write('\t\t'+temp1+' = '+temp[1][temp1]+'\n')
                else:
                    raise Exception
                system.put(temp1)
        else:
            f.write('\t\telse\n')
            f.write("\t\t\tdisp('Cannot find a valid successor, environment assumption is like to be violated')\n")
            for l in range(system.qsize()):
                temp1=system.get()
                if verbosem==1:
                    f.write('\t\t\t'+temp1+' = '+temp[1][temp1]+';\n')
                elif verbosem==0:
                    f.write('\t\t\t'+temp1+' = '+temp[1][temp1]+'\n')
                else:
                    raise Exception
                system.put(temp1)
            f.write('\t\tend\n')
        queue.put(temp)

    #the last case is an otherwise statement
    f.write('\totherwise\n')
    f.write("\t\tdisp('Cannot find a valid successor, environment assumption is like to be violated')\n")
    for l in range(system.qsize()):
        temp1=system.get()
        if verbosem==1:
            f.write('\t\t'+temp1+' = 0;\n')
        elif verbosem==0:
            f.write('\t\t'+temp1+' = 0\n')
        else:
            raise Exception
        system.put(temp1)
    f.write('end')

queue=Queue.Queue()
queue1=Queue.Queue()
queue2=Queue.Queue()
enviroment=Queue.Queue()
system=Queue.Queue()
try:
    load_file(sys.argv[1]+'.aut')
    read_variables(sys.argv[1]+'.smv')
    q=question('Shall there be a semicolon printed after each variable assignment? [Y/n]')
    q2=question('Shall the script exclude no successors? [Y/n]')
    if q:
        verbosem=1
    else:
        verbosem=0
    if not os.path.isfile(sys.argv[2]+'.m'):
        f=open(sys.argv[2]+'.m','w')
        write_startline(enviroment,system,f)
        if q2:
            for i in range(queue.qsize()):
                temp=queue.get()
                temp1=queue1.get()
                if not temp[2] == []:
                    queue.put(temp)
                    queue1.put(temp1)
            write_case_no(enviroment,system,f,verbosem)
        else:
            write_case(enviroment,system,f,verbosem)
        f.close()
        if queue.get()[0]==-1:
            raise IOError
        print('MATLAB script written to '+sys.argv[2]+'.m'+' with success\n')
    else:
        print('Enter a matlab filename that does not exist.')
except IOError:
    print('Enter correct filename for a TuLiP generated controller, '+ \
        'aut and\nsmv file must have the same name')
except IndexError:
    print('Usage: aut2simulink.py JTLV-AUT-FILE MATLAB-FILE\n\n'+ \
          '    aut and smv file must have the same name.\n'+ \
          '    Do not include file extensions.')



"""Extra tests for interface to `omega.games.gr1`.

Compared to omega_interface_test.py, tests here can depend on
configurations of `omega` or `dd` that involve configurations beyond
the required dependencies of TuLiP. E.g., dd can be built with CUDD.
"""
import logging

from tulip.spec import form
from tulip.interfaces import omega as omega_int

from omega_interface_test import grspec_1


logging.getLogger('tulip').setLevel('ERROR')
logging.getLogger('astutils').setLevel('ERROR')
logging.getLogger('omega').setLevel('ERROR')
log = logging.getLogger('omega.games')
log.setLevel('WARNING')
log.addHandler(logging.StreamHandler())


def test_synthesis_cudd():
    sp = grspec_1()
    h = omega_int.synthesize_enumerated_streett(sp, use_cudd=True)
    assert h is not None
    n = len(h)
    assert n == 25, n


def test_is_circular_cudd():
    f = form.GRSpec()
    f.sys_vars['y'] = 'bool'
    f.env_prog = ['y']
    f.sys_prog = ['y']
    triv = omega_int.is_circular(f, use_cudd=True)
    assert triv, triv



#!/usr/bin/env python
"""Tests for gr1 fragment untilities."""
import logging
logging.basicConfig(level=logging.DEBUG)
logging.getLogger('tulip.ltl_parser_log').setLevel(logging.WARNING)
logging.getLogger('tulip.spec.form').setLevel(logging.WARNING)
logging.getLogger('omega').setLevel(logging.WARNING)
from nose.tools import assert_raises
from tulip import spec, synth
from tulip.spec import parser, transformation
from tulip.spec import gr1_fragment as gr1


def test_split_gr1():
    # init
    f = '(x > 0) & (y + 1 < 2)'
    d = gr1.split_gr1(f)
    assert d['init'] == ['( x > 0 ) & ( ( y + 1 ) < 2 )'], d
    assert d['G'] == [''], d
    assert d['GF'] == [], d
    # safety
    f = '[]((x > 0) & (z = 3 + y))'
    d = gr1.split_gr1(f)
    assert d['init'] == [''], d
    assert d['G'] == ['( ( x > 0 ) & ( z = ( 3 + y ) ) )'], d
    assert d['GF'] == [], d
    # recurrence
    f = '[]<>(x > 0)'
    d = gr1.split_gr1(f)
    assert d['init'] == [''], d
    assert d['G'] == [''], d
    assert d['GF'] == ['( x > 0 )'], d
    # all together
    f = (
        '(x > 0) & (y + 1 < 2) & '
        '[]( (X y) > 0) & '
        '[]<>((z - x <= 0) | (p -> q))')
    d = gr1.split_gr1(f)
    assert d['init'] == ['( x > 0 ) & ( ( y + 1 ) < 2 )'], d
    assert d['G'] == ['( ( X y ) > 0 )'], d
    assert d['GF'] == ['( ( ( z - x ) <= 0 ) | ( p -> q ) )'], d
    # not in fragment
    with assert_raises(AssertionError):
        gr1.split_gr1('[]( [] p )')
    with assert_raises(AssertionError):
        gr1.split_gr1('<>( [] p )')
    with assert_raises(AssertionError):
        gr1.split_gr1('(X p ) & ( [] p )')
    with assert_raises(AssertionError):
        gr1.split_gr1('[]<> ( x & (X y) )')


def test_has_operator():
    t = parser.parse(' [](x & y) ')
    g = transformation.Tree.from_recursive_ast(t)
    assert gr1.has_operator(t, g, {'&'}) == '&'
    assert gr1.has_operator(t, g, {'G'}) == 'G'


def test_name_conflict():
    assert_raises(ValueError, gr1.stability_to_gr1, 'a', aux='a')


def test_stability():
    s = gr1.stability_to_gr1('p', aux='a')

    assert isinstance(s, spec.GRSpec)
    assert 'aux' not in s.sys_vars
    assert 'a' in s.sys_vars
    assert 'p' in s.sys_vars

    s.moore = False
    s.plus_one = False
    s.qinit = '\A \E'

    # p && X[]!p
    s0 = spec.GRSpec(
        sys_vars={'p'}, sys_init={'p'},
        sys_safety={'p -> X !p',
                    '!p -> X !p'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s0)

    # !p && X[]p
    s1 = spec.GRSpec(
        sys_vars={'p'}, sys_init={'!p'},
        sys_safety={'!p -> X p',
                    'p -> X p'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s1)

    # []<>p && []<>!p
    s2 = spec.GRSpec(
        sys_vars={'p'},
        sys_prog={'p', '!p'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s2)

    # env b can prevent !p, but has tp <> become !b,
    # releasing sys to set p
    #
    # env: b && []<>!b
    # sys: !p && []((b && !p) -> X!p)
    s3 = spec.GRSpec(
        env_vars={'b'}, env_init={'b'},
        env_prog={'!b'},
        sys_vars={'p'}, sys_init={'!p'},
        sys_safety={'(b && !p) -> X !p'},
        moore=False,
        plus_one=False,
        qinit='\A \E')

    assert synth.is_realizable('omega', s | s3)

    s3.env_prog = []
    assert not synth.is_realizable('omega', s | s3)

    # s4 = s | s3
    # print(s4.pretty() )
    # mealy = synth.synthesize('omega', s4)
    # mealy.save()


def test_response():
    s = gr1.response_to_gr1('p', 'q')

    assert isinstance(s, spec.GRSpec)
    assert 'p' in s.sys_vars
    assert 'q' in s.sys_vars

    s.moore = False
    s.plus_one = False
    s.qinit = '\A \E'

    # p && []!q
    s0 = spec.GRSpec(
        sys_vars={'p', 'q'},
        sys_init={'p'},
        sys_safety={'!q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s0)

    # []!p && []!q
    s1 = spec.GRSpec(
        sys_vars={'p', 'q'},
        sys_safety={'!p && !q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s1)

    # p && q
    s2 = spec.GRSpec(
        sys_vars={'p', 'q'},
        sys_init={'p && q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s2)

    # alternating p, alternating q
    s3 = spec.GRSpec(
        sys_vars={'p', 'q'},
        sys_safety={
            'p -> X !p',
            '!p -> X p',
            'p -> X q',
            'q -> X ! q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s3)
    # print((s | s2).pretty() )


def test_eventually():
    s = gr1.eventually_to_gr1('p', aux='c')

    assert isinstance(s, spec.GRSpec)
    assert 'aux' not in str(s)
    assert 'c' in s.sys_vars
    assert 'p' in s.sys_vars

    s.moore = False
    s.plus_one = False
    s.qinit = '\A \E'

    # []!p
    s0 = spec.GRSpec(
        sys_vars={'p'},
        sys_safety={'!p'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s0)

    # !p && []<>p && []<>!p
    s1 = spec.GRSpec(
        sys_vars={'p'},
        sys_init={'!p'},
        sys_prog={'!p', 'p'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s1)

    # s2 = s | s1
    # print(s4.pretty() )
    # mealy = synth.synthesize('omega', s4)
    # mealy.save()


def test_until():
    s = gr1.until_to_gr1('p', 'q', aux='c')

    assert isinstance(s, spec.GRSpec)
    assert 'aux' not in str(s)
    assert 'c' in s.sys_vars
    assert 'p' in s.sys_vars
    assert 'q' in s.sys_vars

    s.moore = False
    s.plus_one = False
    s.qinit = '\A \E'

    # []!q
    s0 = spec.GRSpec(
        sys_vars={'q'},
        sys_safety={'!q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s0)

    # !q && <>q
    s1 = spec.GRSpec(
        sys_vars={'q'},
        sys_init={'!q'},
        sys_prog={'q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert synth.is_realizable('omega', s | s1)

    # !q && []!p && <>q
    s1 = spec.GRSpec(
        sys_vars={'q'},
        sys_init={'!q'},
        sys_safety={'!p'},
        sys_prog={'q'},
        moore=False,
        plus_one=False,
        qinit='\A \E'
    )
    assert not synth.is_realizable('omega', s | s1)


if __name__ == '__main__':
    test_response()



"""Tests for transys.transys (part of transys subpackage)"""
import logging
logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger(__name__)
logging.getLogger('tulip.transys.products').setLevel(logging.DEBUG)
from tulip import transys as trs
from tulip.transys.mathset import MathSet, PowerSet


def ts_test():
    ts = trs.FTS()

    ts.states.add('s0')
    assert('s0' in ts)
    assert('s0' in ts.node)
    assert('s0' in ts.states)

    states = {'s0', 's1', 's2', 's3'}
    ts.states.add_from(states)
    assert(set(ts.states) == states)

    ts.transitions.add('s0', 's1')
    ts.transitions.add_from([('s1', 's2'), ('s2', 's3'), ('s3', 's0')])

    ts.states.initial.add('s0')
    ts.states.initial.add_from({'s0', 's1'})

    ts.atomic_propositions.add('p')
    assert(set(ts.atomic_propositions) == {'p'})

    ts.states['s0']['ap'] = {'p'}
    ts.states['s1']['ap'] = set()
    ts.states['s2']['ap'] = set()
    ts.states['s3']['ap'] = set()
    assert(ts.states['s0']['ap'] == {'p'})

    for state in {'s1', 's2', 's3'}:
        assert(ts.states[state]['ap'] == set() )

    logger.debug(ts)
    return ts


def ba_test():
    ba = trs.BA()

    aps = ['p']
    ba.atomic_propositions |= {'p'}
    assert('p' in ba.atomic_propositions)
    assert(ba.atomic_propositions == MathSet(aps) )
    assert(ba.alphabet == PowerSet(aps) )


    ba.states.add_from({'q0', 'q1'})
    assert(set(ba.states) == {'q0', 'q1'})

    ba.states.initial.add('q0')
    assert(set(ba.states.initial) == {'q0'})

    ba.states.accepting.add('q1')
    assert(set(ba.states.accepting) == {'q1'})

    ba.transitions.add('q0', 'q1', letter={'p'})
    ba.transitions.add('q1', 'q1', letter={'p'})
    ba.transitions.add('q1', 'q0', letter=set() )
    ba.transitions.add('q0', 'q0', letter=set() )

    logger.debug(ba)
    ba.save('ba.pdf')
    return ba


def ba_ts_prod_test():
    ts = ts_test()
    ba = ba_test()
    ba_ts = trs.products.ba_ts_sync_prod(ba, ts)

    check_prodba(ba_ts)
    (ts_ba, persistent) = trs.products.ts_ba_sync_prod(ts, ba)


    states = {('s0', 'q1'), ('s1', 'q0'),
              ('s2', 'q0'), ('s3', 'q0')}
    assert(set(ts_ba.states) == states)
    assert(persistent == {('s0', 'q1')} )


    ba_ts.save('prod.pdf')
    return ba_ts


def check_prodba(ba_ts):
    states = {('s0', 'q1'), ('s1', 'q0'),
              ('s2', 'q0'), ('s3', 'q0')}
    assert(set(ba_ts.states) == states)

    assert(set(ba_ts.states.initial) == {('s0', 'q1'), ('s1', 'q0')})

    assert(

        ba_ts.transitions.find(
            [('s0', 'q1')], [('s1', 'q0')]
        )[0][2]['letter'] == set()
    )

    for si, sj in [('s1', 's2'), ('s2', 's3')]:
        assert(
            ba_ts.transitions.find(
                [(si, 'q0')], [(sj, 'q0')]
            )[0][2]['letter'] == set()
        )

    assert(
        ba_ts.transitions.find(
            [('s3', 'q0')], [('s0', 'q1')]
        )[0][2]['letter'] == {'p'}
    )

def on_the_fly_test():
    ba = ba_test()
    ts = ts_test()
    prodba = trs.OnTheFlyProductAutomaton(ba, ts)
    assert(set(prodba.states) == {('s0', 'q1'), ('s1', 'q0')})

    prodba.save('prodba_initialized.pdf')
    prodba.add_all_states()
    check_prodba(prodba)
    prodba.save('prodba_full.pdf')



"""
Export the TuLiP output as an FMU
"""
import uuid
import numpy
import os
from ctrlexport import mealyexport
from pppexport import pppexport
from poly2str import matrix2str, polytope2str


def exportXML(ctrl, pwa, uid, filename="modelDescription"):
    """Generate the modelDescription.xml for the FMU

    @param ctrl: tulip.transys.machines.MealyMachine
    @param pwa: tulip.abstract,discretization.AbstractPwa
    @param uid: the uid the FMU (uuid.UUID)
    @param filename: a string of the export filename
    @rtype: None
    """
    f = open(filename+".xml", 'w')

    f.write('<fmiModelDescription\n')
    f.write('    description="Discrete Time Controller"\n')
    f.write('    fmiVersion="1.5"\n')
    f.write('    guid="{' + str(uid)+'}"\n')
    f.write('    modelName="TuLiPFMU">\n\n')

    f.write('<CoSimulation modelIdentifier="TuLiPFMU" \
            canHandleVariableCommunicationStepSize="true" \
            canHandleEvents="true" \
            canProvideMaxStepSize="true"/>\n')

    f.write('<ModelVariables>\n')
    # number of state
    n = pwa.pwa.B.shape[0]
    # number of control
    m = pwa.pwa.B.shape[1]
    # output real variable: the control output
    for i in range(0, m):
        f.write('<ScalarVariable name="u'+str(i)+'" \
                valueReference="'+str(i)+'" \
                description="output" \
                causality="output">')
        f.write('<Real/>\n')
        f.write('</ScalarVariable>\n')

    # input real variable: the current state of the system
    for i in range(0, n):
        f.write('<ScalarVariable name="y'+str(i)+'" \
                valueReference="'+str(i+m)+'" \
                description="input" \
                causality="input">')
        f.write('<Real/>\n')
        f.write('</ScalarVariable>\n')

    # input discrete variable
    i = 0
    for inputname, inputset in ctrl.inputs.items():
        f.write('<ScalarVariable name="'+inputname+'" \
                valueReference="'+str(i+m+n)+'" \
                description="input" \
                causality="input">')
        f.write('<Integer/>')
        f.write('</ScalarVariable>')
        i = i+1

    f.write('</ModelVariables>\n')
    f.write('</fmiModelDescription>\n')
    f.close()


def exportFMUheader(uid, tick, filename="TuLiPFMU"):
    """Export the TuLiPFMU.h file

    @param uid: the uid of the FMU (uuid.UUID)
    @param tick: the time for one MPC step
    @param filename: a string of the export filename
    @rtype: None
    """
    f = open("include/" + filename + ".h", 'w')
    f.write('#ifndef __TULIPFMU_H__\n')
    f.write('#define __TULIPFMI_H__\n')
    f.write('#define MODEL_IDENTIFIER TuLiPFMU\n')
    f.write('#define MODEL_GUID "{' + str(uid) + '}"\n')
    f.write('#define EPSILON 1e-9\n')
    f.write('#define TICK_PERIOD ' + str(tick) + '\n')
    f.write('#endif\n')
    f.close()


def exportSysData(ctrl, pwa, initState, initRegion, filename="data"):
    """Export the data.c file

    @param ctrl: tulip.transys.machines.MealyMachine
    @param pwa: tulip.abstract,discretization.AbstractPwa
    @param initState: the initial state of the continuous system (numpy.array)
    @param initRegion: the initial region which the initial state belongs to
    @param filename: a string of the export filename
    @rtype: None
    """
    f = open("sources/" + filename + ".c", 'w')
    A = pwa.pwa.A
    B = pwa.pwa.B
    n = B.shape[0]
    # assuming full state observation, hence m = n
    m = B.shape[0]
    p = B.shape[1]
    f.write('#include "data.h"\n')
    f.write('idxint n = ' + str(n) + ';\n')
    f.write('idxint m = ' + str(m) + ';\n')
    f.write('idxint p = ' + str(p) + ';\n')
    f.write('pfloat A[] = {' + matrix2str(A) + '};\n')
    f.write('pfloat B[] = {' + matrix2str(B) + '};\n')
    f.write('idxint totalSteps = ' + str(pwa.disc_params['N']) + ';\n')
    f.write('Polytope *input_bound;\n')

    f.write(polytope2str(pwa.pwa.Uset, 'pu'))
    f.write('\nvoid init_input_bound(void)\n{\n')
    f.write('   input_bound=create_poly(puk,pul,puA,pub,pucenter);\n')
    f.write('}\n\n')

    f.write('void free_input_bound(void)\n{\n')
    f.write('		free(input_bound);\n')
    f.write('}\n')

    f.write('pfloat x0[] = {' + matrix2str(initState) + '};\n')
    f.write('idxint dRegion0 = ' + str(initRegion) + ';\n')
    f.close()


def exportFMU(ctrl, pwa, initState, initRegion, tick=1):
    """Generate the fmu

    @param ctrl: tulip.transys.machines.MealyMachine
    @param pwa: tulip.abstract,discretization.AbstractPwa
    @param initState: the initial state of the continuous system (numpy.array)
    @param initRegion: the initial region which the initial state belongs to
    @param tick: the time for one MPC step
    @rtype: None
    """
    uid = uuid.uuid1()
    exportXML(ctrl, pwa, uid)
    exportFMUheader(uid, tick)
    exportSysData(ctrl, pwa, initState, initRegion)
    pppexport(pwa.ppp)
    mealyexport(ctrl)
    os.system("make fmu")



#!/usr/bin/env python
"""
Usage: genrandt.py [-p] [-i FILE] [-t N] [-g G] [-b] [H W]

will generate a random gridworld of height H and width W (default is
5x10), with N trolls (default 1) at random positions, G goals (default 2)
at random positions, and print the resulting specification.

Troll region radii are set using the variable TROLL_RADIUS (default 1).

If the flag "-p" is given, then generate a plot of the grid.  If the -i
flag is given in addition to -p, then save the plot to a PDF FILE.  If the
flag -b is given, instead of integer-valued variables, use representation
where there is one boolean variable in the specification per grid cell.
"""
from __future__ import print_function

import sys
import matplotlib as mpl
mpl.use('agg')  # change the backend as available in your environment
import matplotlib.pyplot as plt
import tulip.gridworld as gw


TROLL_RADIUS = 1

if len(sys.argv) > 11 or "-h" in sys.argv:
    print("Usage: genrandt.py [-p] [-i FILE] [-t N] [-g G] [-b] [H W]")
    sys.exit(1)

if "-b" in sys.argv:
    nonbool = False
    sys.argv.remove("-b")
else:
    nonbool = True

try:
    targ_ind = sys.argv.index("-t")
    if targ_ind > len(sys.argv)-2:
        print("Invalid use of -t flag.  Try \"-h\"")
        sys.exit(1)
except ValueError:
    targ_ind = -1
if targ_ind < 0:
    N = 1
else:
    N = int(sys.argv[targ_ind+1])

try:
    garg_ind = sys.argv.index("-g")
    if garg_ind > len(sys.argv)-2:
        print("Invalid use of -g flag.  Try \"-h\"")
        sys.exit(1)
except ValueError:
    garg_ind = -1
if garg_ind < 0:
    num_goals = 2
else:
    num_goals = int(sys.argv[garg_ind+1])

if "-p" in sys.argv:
    print_pretty = True
    sys.argv.remove("-p")
    try:
        iarg_ind = sys.argv.index("-i")+1
        if iarg_ind > len(sys.argv)-1:
            print("Invalid use of -i flag.  Try \"-h\"")
            sys.exit(1)
    except ValueError:
        iarg_ind = -1
else:
    print_pretty = False

if len(sys.argv) >= 3 and sys.argv[-2][0] != "-":
    (height, width) = (int(sys.argv[-2]), int(sys.argv[-1]))
else:
    (height, width) = (5, 10)

Z, troll_list = gw.random_world((height, width),
                                wall_density=0.2,
                                num_init=1,
                                num_goals=num_goals, num_trolls=N)
for i in range(len(troll_list)):
    troll_list[i] = (troll_list[i][0], TROLL_RADIUS)

print(Z.pretty(show_grid=True, line_prefix="## "))

print(Z.dumps(line_prefix="# "))
(spec, moves_N) = gw.add_trolls(Z, troll_list)
print(spec.pretty())

if print_pretty:
    Z.plot(font_pt=0, troll_list=troll_list)
    if iarg_ind == -1:
        plt.show()
    else:
        plt.savefig(sys.argv[iarg_ind])



#!/usr/bin/env python
"""
Tests for the interface with gr1c.
"""
import logging
logging.basicConfig(level=logging.DEBUG)
logging.getLogger('tulip.spec.lexyacc').setLevel(logging.WARNING)
import networkx as nx
from nose.tools import raises
import os
from tulip.spec import GRSpec, translate
from tulip.interfaces import gr1c


REFERENCE_SPECFILE = """
# For example, regarding states as bitvectors, 1011 is not in winning
# set, while 1010 is. (Ordering is x ze y zs.)

ENV: x ze;
SYS: y zs;

ENVINIT: x & !ze;
ENVTRANS: [] (zs -> ze') & []((!ze & !zs) -> !ze');
ENVGOAL: []<>x;

SYSINIT: y;
SYSTRANS:;
SYSGOAL: []<>y&x & []<>!y & []<> !ze;
"""

REFERENCE_AUTXML = """<?xml version="1.0" encoding="UTF-8"?>
<tulipcon xmlns="http://tulip-control.sourceforge.net/ns/1" version="1">
  <env_vars><item key="x" value="boolean" /></env_vars>
  <sys_vars><item key="y" value="boolean" /></sys_vars>
  <spec><env_init></env_init><env_safety></env_safety><env_prog></env_prog><sys_init></sys_init><sys_safety></sys_safety><sys_prog></sys_prog></spec>
  <aut type="basic">
    <node>
      <id>0</id><anno></anno><child_list> 1 2</child_list>
      <state><item key="x" value="1" /><item key="y" value="0" /></state>
    </node>
    <node>
      <id>1</id><anno></anno><child_list> 1 2</child_list>
      <state><item key="x" value="0" /><item key="y" value="0" /></state>
    </node>
    <node>
      <id>2</id><anno></anno><child_list> 1 0</child_list>
      <state><item key="x" value="1" /><item key="y" value="1" /></state>
    </node>
  </aut>
  <extra></extra>
</tulipcon>
"""

REFERENCE_AUTJSON_smallbool = """
{"version": 1,
 "gr1c": "0.10.2",
 "date": "2015-10-10 16:56:17",
 "extra": "",

 "ENV": [{"x": "boolean"}],
 "SYS": [{"y": "boolean"}],

 "nodes": {
"0x1E8FA40": {
    "state": [0, 0],
    "mode": 0,
    "rgrad": 1,
    "initial": false,
    "trans": ["0x1E8FA00"] },
"0x1E8FA00": {
    "state": [1, 1],
    "mode": 1,
    "rgrad": 1,
    "initial": false,
    "trans": ["0x1E8FA40"] },
"0x1E8F990": {
    "state": [0, 1],
    "mode": 0,
    "rgrad": 1,
    "initial": true,
    "trans": ["0x1E8FA00"] }
}}
"""


class basic_test:
    def setUp(self):
        self.f_un = GRSpec(
            env_vars="x",
            sys_vars="y",
            env_init="x",
            env_prog="x",
            sys_init="y",
            sys_safety=["y -> X(!y)", "!y -> X(y)"],
            sys_prog="y && x",
            moore=False,
            plus_one=False,
            qinit='\A \E')
        self.dcounter = GRSpec(
            env_init=['True'],
            sys_vars={"y": (0, 5)},
            sys_init=["y=0"],
            sys_prog=["y=0", "y=5"],
            moore=False,
            plus_one=False,
            qinit='\A \E')

    def tearDown(self):
        self.f_un = None
        self.dcounter = None

    def test_check_syntax(self):
        assert gr1c.check_syntax(REFERENCE_SPECFILE)
        assert not gr1c.check_syntax("foo")

    def test_to_gr1c(self):
        assert gr1c.check_syntax(translate(self.f_un, 'gr1c'))
        assert gr1c.check_syntax(translate(self.dcounter, 'gr1c'))

    def test_check_realizable(self):
        assert not gr1c.check_realizable(self.f_un)
        self.f_un.sys_safety = []
        assert gr1c.check_realizable(self.f_un)
        self.f_un.qinit = '\A \A'
        self.f_un.env_init = ['x', 'y = 0', 'y = 5']
        self.f_un.sys_init = list()
        assert gr1c.check_realizable(self.f_un)
        assert gr1c.check_realizable(self.dcounter)
        self.dcounter.qinit = '\A \A'
        self.dcounter.sys_init = list()
        assert gr1c.check_realizable(self.dcounter)

    def test_synthesize(self):
        self.f_un.sys_safety = list()  # Make it realizable
        g = gr1c.synthesize(self.f_un)
        assert g is not None
        assert len(g.env_vars) == 1 and 'x' in g.env_vars
        assert len(g.sys_vars) == 1 and 'y' in g.sys_vars

        g = gr1c.synthesize(self.dcounter)
        assert g is not None
        assert len(g.env_vars) == 0
        assert len(g.sys_vars) == 1 and 'y' in g.sys_vars
        assert len(g) == 2

        # In the notation of gr1c SYSINIT: True;, so the strategy must
        # account for every initial state, i.e., for y=0, y=1, y=2, ...
        self.dcounter.qinit = '\A \A'
        self.dcounter.sys_init = list()
        g = gr1c.synthesize(self.dcounter)
        assert g is not None
        print g
        assert len(g.env_vars) == 0
        assert len(g.sys_vars) == 1 and 'y' in g.sys_vars
        assert len(g) == 6, len(g)


class GR1CSession_test:
    def setUp(self):
        self.spec_filename = "trivial_partwin.spc"
        with open(self.spec_filename, "w") as f:
            f.write(REFERENCE_SPECFILE)
        self.gs = gr1c.GR1CSession("trivial_partwin.spc",
                                   env_vars=["x", "ze"],
                                   sys_vars=["y", "zs"])

    def tearDown(self):
        self.gs.close()
        os.remove(self.spec_filename)

    def test_numgoals(self):
        assert self.gs.numgoals() == 3

    def test_reset(self):
        assert self.gs.reset()

    def test_getvars(self):
        vars_str = self.gs.getvars()
        vars_list = [vi.strip() for vi in vars_str.split(",")]
        assert vars_list == ["x (0)", "ze (1)", "y (2)", "zs (3)"]

    def test_getindex(self):
        assert self.gs.getindex({"x": 0, "y": 0, "ze": 0, "zs": 0}, 0) == 1
        assert self.gs.getindex({"x": 0, "y": 0, "ze": 0, "zs": 0}, 1) == 1

    def test_iswinning(self):
        assert self.gs.iswinning({"x": 1, "y": 1, "ze": 0, "zs": 0})
        assert not self.gs.iswinning({"x": 1, "y": 1, "ze": 0, "zs": 1})

    def test_env_next(self):
        assert (self.gs.env_next({"x": 1, "y": 1, "ze": 0, "zs": 0}) ==
                [{'x': 0, 'ze': 0}, {'x': 1, 'ze': 0}])
        assert (self.gs.env_next({"x": 1, "y": 1, "ze": 0, "zs": 1}) ==
                [{'x': 0, 'ze': 1}, {'x': 1, 'ze': 1}])

    def test_sys_nexta(self):
        assert (
            self.gs.sys_nexta(
                {"x": 1, "y": 1, "ze": 0, "zs": 0},
                {"x": 0, "ze": 0}
            ) == [
                {'y': 0, 'zs': 0},
                {'y': 0, 'zs': 1},
                {'y': 1, 'zs': 0},
                {'y': 1, 'zs': 1}
            ])

    def test_sys_nextfeas(self):
        assert self.gs.sys_nextfeas(
            {"x": 1, "y": 1, "ze": 0, "zs": 0},
            {"x": 0, "ze": 0}, 0) == [{'y': 0, 'zs': 0}, {'y': 1, 'zs': 0}]


def test_aut_xml2mealy():
    g = gr1c.load_aut_xml(REFERENCE_AUTXML)
    assert g.env_vars == {"x": "boolean"}
    assert g.sys_vars == {"y": "boolean"}
    print(g.nodes())
    assert len(g) == 3

def test_load_aut_json():
    g = gr1c.load_aut_json(REFERENCE_AUTJSON_smallbool)
    assert g.env_vars == dict(x='boolean'), (g.env_vars)
    assert g.sys_vars == dict(y='boolean'), (g.sys_vars)
    # `REFERENCE_AUTJSON_smallbool` defined above
    h = nx.DiGraph()
    nodes = {0: '0x1E8FA40', 1: '0x1E8FA00', 2: '0x1E8F990'}
    h.add_node(nodes[0], state=dict(x=0, y=0),
               mode=0, rgrad=1, initial=False)
    h.add_node(nodes[1], state=dict(x=1, y=1),
               mode=1, rgrad=1, initial=False)
    h.add_node(nodes[2], state=dict(x=0, y=1),
               mode=0, rgrad=1, initial=True)
    edges = [(nodes[0], nodes[1]),
             (nodes[1], nodes[0]),
             (nodes[2], nodes[1])]
    h.add_edges_from(edges)
    # compare
    for u, d in h.nodes_iter(data=True):
        assert u in g, (u, g.nodes())
        d_ = g.node[u]
        for k, v in d.iteritems():
            v_ = d_.get(k)
            assert v_ == v, (k, v, v_, d, d_)
    h_edges = set(h.edges_iter())
    g_edges = set(g.edges_iter())
    assert h_edges == g_edges, (h_edges, g_edges)


@raises(ValueError)
def synth_init_illegal_check(init_option):
    spc = GRSpec(moore=False, plus_one=False, qinit=init_option)
    gr1c.synthesize(spc)


def synth_init_illegal_test():
    for init_option in ["Caltech", 1]:
        yield synth_init_illegal_check, init_option


@raises(ValueError)
def realiz_init_illegal_check(init_option):
    spc = GRSpec(moore=False, plus_one=False, qinit=init_option)
    gr1c.check_realizable(spc)


def realiz_init_illegal_test():
    for init_option in ["Caltech", 1]:
        yield realiz_init_illegal_check, init_option



#!/usr/bin/env python
#
# WARNING: This example may not yet be working.  Please check again in
#          the upcoming release.
#
"""
Continuous example with MATLAB export at the end. Mostly identical to
examples/robot_planning/continuous.py
"""
#
# Note: This code is commented to allow components to be extracted into
# the tutorial that is part of the users manual.  Comments containing
# strings of the form @label@ are used for this purpose.
#
# @import_section@
import sys
sys.path.append('../')
import numpy as np
import tomatlab
from tulip import spec, synth, hybrid
from polytope import box2poly
from tulip.abstract import prop2part, discretize, find_controller
# @import_section_end@

visualize = False
from tulip.abstract.plot import plot_partition

# @dynamics_section@
# Problem parameters
input_bound = 1.0
uncertainty = 0.01

# Continuous state space
cont_state_space = box2poly([[0., 3.], [0., 2.]])

# Continuous dynamics
A = np.array([[1.0, 0.], [ 0., 1.0]])
B = np.array([[0.1, 0.], [ 0., 0.1]])
E = np.array([[1,0], [0,1]])

# Available control, possible disturbances
U = input_bound *np.array([[-1., 1.], [-1., 1.]])
W = uncertainty *np.array([[-1., 1.], [-1., 1.]])

# Convert to polyhedral representation
U = box2poly(U)
W = box2poly(W)

# Construct the LTI system describing the dynamics
sys_dyn = hybrid.LtiSysDyn(A, B, E, None, U, W, cont_state_space)
# @dynamics_section_end@

# @partition_section@
# Define atomic propositions for relevant regions of state space
cont_props = {}
cont_props['home'] = box2poly([[0., 1.], [0., 1.]])
cont_props['lot'] = box2poly([[2., 3.], [1., 2.]])

# Compute the proposition preserving partition of the continuous state space
cont_partition = prop2part(cont_state_space, cont_props)
plot_partition(cont_partition, show=visualize)
# @partition_section_end@

# @discretize_section@
# Given dynamics & proposition-preserving partition, find feasible transitions
disc_params = {'closed_loop':True, 'N':8, 'min_cell_volume':0.1,
               'plotit':visualize, 'conservative':False}
disc_dynamics = discretize(cont_partition, sys_dyn, **disc_params)
# @discretize_section_end@

"""Visualize transitions in continuous domain (optional)"""
plot_partition(disc_dynamics.ppp, disc_dynamics.ts,
               disc_dynamics.ppp2ts, show=visualize)

"""Specifications"""
# Environment variables and assumptions
env_vars = {'park'}
env_init = set()                # empty set
env_prog = '!park'
env_safe = set()                # empty set

# System variables and requirements
sys_vars = {'X0reach'}
sys_init = {'X0reach'}
sys_prog = {'home'}               # []<>home
sys_safe = {'(X(X0reach) <-> lot) || (X0reach && !park)'}
sys_prog |= {'X0reach'}

# Create the specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)

# @synthesize_section@
"""Synthesize"""
ctrl = synth.synthesize('gr1c', specs,
                        sys=disc_dynamics.ts, ignore_sys_init=True)
# Unrealizable spec ?
if ctrl is None:
    sys.exit()

# Export Simulink Model
tomatlab.export('robot_continuous.mat', ctrl, sys_dyn, disc_dynamics,
                disc_params)



#!/usr/bin/env python
"""Controller synthesis for system with piecewise-affine continuous dynamics."""
# This example is an extension of `robot_continuous.py`
# by Petter Nilsson and Nok Wongpiromsarn.
# Necmiye Ozay, August 26, 2012
import numpy as np
from polytope import box2poly
from tulip.abstract import discretize
from tulip.abstract import prop2part
from tulip.abstract.plot import plot_strategy
from tulip.hybrid import LtiSysDyn
from tulip.hybrid import PwaSysDyn
from tulip import spec
from tulip import synth


# set to `True` if `matplotlib.pyplot` is available
plotting = False

# Problem parameters
input_bound = 0.4
uncertainty = 0.05

# Continuous state space
cont_state_space = box2poly([[0., 3.], [0., 2.]])

# Assume, for instance, our robot is traveling on
# a nonhomogenous surface (xy plane),
# resulting in different dynamics at different
# parts of the plane.
#
# Since the continuous state space in this example
# is just xy position, different dynamics in
# different parts of the surface can be modeled
# using LtiSysDyn subsystems subsys0 and subsys1.
#
# Together they comprise a Piecewise Affine System:

# @subsystem0@
def subsys0():
    A = np.array([[1.1052, 0.], [ 0., 1.1052]])
    B = np.array([[1.1052, 0.], [ 0., 1.1052]])
    E = np.array([[1,0], [0,1]])

    U = box2poly([[-1., 1.], [-1., 1.]])
    U.scale(input_bound)

    W = box2poly([[-1., 1.], [-1., 1.]])
    W.scale(uncertainty)

    dom = box2poly([[0., 3.], [0.5, 2.]])

    sys_dyn = LtiSysDyn(A, B, E, None, U, W, dom)
    #sys_dyn.plot()

    return sys_dyn
# @subsystem0_end@

# @subsystem1@
def subsys1():
    A = np.array([[0.9948, 0.], [0., 1.1052]])
    B = np.array([[-1.1052, 0.], [0., 1.1052]])
    E = np.array([[1, 0], [0, 1]])

    U = box2poly([[-1., 1.], [-1., 1.]])
    U.scale(input_bound)

    W = box2poly([[-1., 1.], [-1., 1.]])
    W.scale(uncertainty)

    dom = box2poly([[0., 3.], [0., 0.5]])

    sys_dyn = LtiSysDyn(A, B, E, None, U, W, dom)
    #sys_dyn.plot()

    return sys_dyn
# @subsystem1_end@

# @pwasystem@
subsystems = [subsys0(), subsys1()]

# Build piecewise affine system from its subsystems
sys_dyn = PwaSysDyn(subsystems, cont_state_space)
if plotting:
    ax = sys_dyn.plot()
    ax.figure.savefig('pwa_sys_dyn.pdf')
# @pwasystem_end@

# Continuous proposition
cont_props = {}
cont_props['home'] = box2poly([[0., 1.], [0., 1.]])
cont_props['lot'] = box2poly([[2., 3.], [1., 2.]])

# Compute the proposition preserving partition
# of the continuous state space
cont_partition = prop2part(cont_state_space, cont_props)
if plotting:
    ax = cont_partition.plot()
    cont_partition.plot_props(ax=ax)
    ax.figure.savefig('spec_ppp.pdf')

disc_dynamics = discretize(
    cont_partition, sys_dyn, closed_loop=True,
    N=8, min_cell_volume=0.1, plotit=plotting, save_img=True,
    cont_props=cont_props)
if plotting:
    ax = disc_dynamics.plot(show_ts=True)
    ax.figure.savefig('abs_pwa.pdf')
    disc_dynamics.ts.save('ts.pdf')

# Specifications

# Environment variables and assumptions
env_vars = {'park'}
env_init = set()                # empty set
env_prog = '!park'
env_safe = set()                # empty set

# System variables and requirements
sys_vars = {'X0reach'}

# []<>home
sys_prog = {'home'}

# [](park -> <> lot)
sys_init = {'X0reach'}
sys_safe = {'X(X0reach) <-> lot || (X0reach && !park)'}
sys_prog |= {'X0reach'}

# Create the specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
specs.moore = True
specs.qinit = '\E \A'

# Synthesize
ctrl = synth.synthesize('omega', specs,
                        sys=disc_dynamics.ts, ignore_sys_init=True)
assert ctrl is not None, 'unrealizable'
if plotting:
    ax = plot_strategy(disc_dynamics, ctrl)
    ax.figure.savefig('pwa_proj_mealy.pdf')

# Save graphical representation of controller for viewing
if not ctrl.save('pwa.png'):
    print(ctrl)

# Simulation



#!/usr/bin/env python
"""
Tests for abstract.prop2partition
"""

from tulip.abstract import prop2part
import polytope as pc
import numpy as np

def prop2part_test():
    state_space = pc.Polytope.from_box(np.array([[0., 2.],[0., 2.]]))

    cont_props = []
    A = []
    b = []

    A.append(np.array([[1., 0.],
                       [-1., 0.],
                       [0., 1.],
                       [0., -1.]]))
    b.append(np.array([[.5, 0., .5, 0.]]).T)
    cont_props.append(pc.Polytope(A[0], b[0]))

    A.append(np.array([[1., 0.],
                       [-1., 0.],
                       [0., 1.],
                       [0., -1.]]))
    b.append(np.array([[2., -1.5, 2., -1.5]]).T)
    cont_props.append(pc.Polytope(A[1], b[1]))

    cont_props_dict = {"C"+str(i) : pc.Polytope(A[i], b[i]) for i in range(2)}

    mypartition = prop2part(state_space, cont_props_dict)
    print(mypartition)

    ref_adjacency = np.array([[1,0,1],[0,1,1],[1,1,1]])
    assert np.all(mypartition.adj.todense() == ref_adjacency)

    assert len(mypartition.regions) == 3

    for reg in mypartition.regions[0:2]:
        assert len(reg.props) == 1
        assert len(reg) == 1

        assert cont_props_dict == mypartition.prop_regions

    assert len(mypartition.regions[2].props) == 0

    assert len(mypartition.regions[2]) == 3
    dum = state_space.copy()
    for reg in mypartition.regions[0:2]:
        dum = dum.diff(reg)
    assert pc.is_empty(dum.diff(mypartition.regions[2]) )
    assert pc.is_empty(mypartition.regions[2].diff(dum) )

    assert(mypartition.preserves_predicates())

    # invalidate it
    mypartition.regions += [pc.Region([pc.Polytope(A[0], b[0])], {})]
    assert(not mypartition.preserves_predicates())



#!/usr/bin/env python
"""Discrete synthesis from a dummy abstraction with mixed switching.

This is an example to demonstrate how the output of a discretization algorithm
that abstracts a switched system might look like,
where the mode of the system depends on a combination of
environment and system controlled variables.

We assume within each mode that the control authority is rich enough to
establish deterministic reachability relations,
through the use of low-level continuous inputs.

We will assume, we have the 6 cell robot example.

     +---+---+---+
     | 3 | 4 | 5 |
     +---+---+---+
     | 0 | 1 | 2 |
     +---+---+---+
"""
# NO, 26 Jul 2013.
import logging

import numpy as np
from scipy import sparse as sp
from tulip import spec
from tulip import synth
from tulip import transys


logging.basicConfig(level=logging.WARNING)
logging.getLogger('tulip.spec').setLevel(logging.WARNING)
logging.getLogger('tulip.synth').setLevel(logging.WARNING)


###########################################
# Hybrid system with 2 env, 2 system modes:
###########################################

sys_hyb = transys.FTS()

# We assume robots ability to transition between cells depends both on
# discrete controlled modes (e.g., gears) and environment modes (e.g., surface
# conditions).

sys_hyb.sys_actions.add_from({'gear0','gear1'})
sys_hyb.env_actions.add_from({'slippery','normal'})

# str states
n = 6
states = ['s'+str(i) for i in xrange(n) ]

sys_hyb.atomic_propositions.add_from(['home','lot'])
state_labels = [{'home'}, set(), set(), set(), set(), {'lot'}]

# Add states and decorate TS with state labels (aka atomic propositions)
for state, label in zip(states, state_labels):
    sys_hyb.states.add(state, ap=label)

# First environment chooses a mode, than the system chooses a mode and within
# each mode there exists a low level controller to take any available transition
# deterministically.

# gear0 basically stops the robot no matter what the enviornment does so
# We take the transitions to be identity.
trans1 = np.eye(6)

sys_hyb.transitions.add_adj(
    sp.lil_matrix(trans1), states,
    sys_actions='gear0', env_actions='normal'
)
sys_hyb.transitions.add_adj(
    sp.lil_matrix(trans1), states,
    sys_actions='gear0', env_actions='slippery'
)

# gear1 dynamics are similar to the environment switching example.
transmat1 = sp.lil_matrix(np.array(
                [[1,1,0,1,0,0],
                 [1,1,1,0,1,0],
                 [0,1,1,0,1,1],
                 [1,0,0,1,1,0],
                 [0,1,0,1,1,1],
                 [0,0,1,0,1,1]]
            ))

sys_hyb.transitions.add_adj(
    transmat1, states, sys_actions='gear1', env_actions='normal'
)

transmat2 = sp.lil_matrix(np.array(
                [[0,0,1,1,0,0],
                 [1,0,1,0,1,0],
                 [1,0,0,0,1,1],
                 [1,0,0,0,0,1],
                 [0,1,0,1,0,1],
                 [0,0,1,1,0,0]]
            ))

sys_hyb.transitions.add_adj(
    transmat2, states, sys_actions='gear1', env_actions='slippery'
)

# This is what is visible to the outside world (and will go into synthesis method)
print(sys_hyb)

#
# Environment variables and specification
#
# The environment can issue a park signal that the robot just respond
# to by moving to the lower left corner of the grid.  We assume that
# the park signal is turned off infinitely often.
#
env_vars = {'park'}
env_init = set()                # empty set
env_prog = {'!park'}
env_safe = set()                # empty set

#
# System specification
#
# The system specification is that the robot should repeatedly revisit
# the upper right corner of the grid while at the same time responding
# to the park signal by visiting the lower left corner.  The LTL
# specification is given by
#
#     []<> home && [](park -> <>lot)
#
# Since this specification is not in GR(1) form, we introduce the
# variable X0reach that is initialized to True and the specification
# [](park -> <>lot) becomes
#
#     [](X (X0reach) <-> lot || (X0reach && !park))
#

# Augment the environmental description to make it GR(1)
#! TODO: create a function to convert this type of spec automatically

# Define the specification
#! NOTE: maybe "synthesize" should infer the atomic proposition from the
# transition system? Or, we can declare the mode variable, and the values
# of the mode variable are read from the transition system.
sys_vars = {'X0reach'}
sys_init = {'X0reach'}
sys_prog = {'home'}               # []<>home
sys_safe = {'(X (X0reach) <-> lot) || (X0reach && !park)'}
sys_prog |= {'X0reach'}

# Possible additional specs
# It is unsafe to "break" (switch to gear0) when road is slippery
sys_safe |= {'(sys_actions = "gear1" && env_actions = "slippery") -> ' +
             'X (sys_actions = "gear1")'}

# to use int actions:
# sys_safe |= {'((act = gear1) && (eact = slippery)) -> X (act = gear1)'}


# Create the specification formulae
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
# Mealy controller (can decide based on `env_vars'`)
specs.moore = False
# Pick initial values for system variables that
# work for all assumed environment initial conditions.
specs.qinit = '\E \A'


# Controller synthesis
#
# At this point we can synthesize the controller using one of the available
# methods.
#
ctrl = synth.synthesize('omega', specs, sys=sys_hyb, ignore_sys_init=True)
assert ctrl is not None, 'unrealizable'
if not ctrl.save('hybrid.png'):
    print(ctrl)



"""Test the management of `tulip.__version__`."""
import imp
import os
import os.path

import git
import mock
from nose.tools import assert_raises
from setuptools.version import pkg_resources

import tulip
import tulip._version


def test_tulip_has_pep440_version():
    """Check that `tulip.__version__` complies to PEP440."""
    version = tulip.__version__
    assert version is not None, version
    version_ = tulip._version.version
    assert version == version_, (version, version_)
    assert_pep440(version)


@mock.patch('git.Repo')
def test_git_version(mock_repo):
    """Mock `git` repository for testing `setup.git_version`."""
    path = os.path.realpath(__file__)
    path = os.path.dirname(path)
    path = os.path.dirname(path)  # parent dir
    path = os.path.join(path, 'setup.py')
    setup = imp.load_source('setup', path)
    # mocking
    version = '0.1.2'
    instance = mock_repo.return_value
    instance.head.commit.hexsha = '0123'
    # dirty repo
    v = setup.git_version(version)
    assert_pep440(v)
    assert 'dev' in v, v
    assert 'dirty' in v, v
    # not dirty, not tagged
    instance.is_dirty.return_value = False
    instance.git.describe.side_effect = git.GitCommandError('0', 0)
    v = setup.git_version(version)
    assert_pep440(v)
    assert 'dev' in v, v
    assert 'dirty' not in v, v
    # tagged as version that matches `setup.py`
    instance.git.describe.side_effect = None
    instance.git.describe.return_value = 'v0.1.2'
    v = setup.git_version(version)
    assert_pep440(v)
    assert v == '0.1.2', v
    # tagged as wrong version
    instance.git.describe.return_value = 'v0.1.3'
    with assert_raises(AssertionError):
        setup.git_version(version)
    # release: no repo
    mock_repo.side_effect = Exception('no repo found')
    with assert_raises(Exception):
        setup.git_version(version)


def assert_pep440(version):
    """Raise `AssertionError` if `version` violates PEP440."""
    v = pkg_resources.parse_version(version)
    assert isinstance(v, pkg_resources.SetuptoolsVersion), v


if __name__ == '__main__':
    test_git_version()



# Copyright (c) 2014, 2015 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
"""Syntactic manipulation of trees."""
import logging
import copy
import os
import warnings
import networkx as nx
from tulip.spec.ast import nodes
from tulip.spec import parser


logger = logging.getLogger(__name__)


class Tree(nx.MultiDiGraph):
    """Abstract syntax tree as a graph data structure.

    Use this as a scaffold for syntactic manipulation.
    It makes traversals and graph rewriting easier,
    so it is preferred to working directly with the
    recursive AST classes.

    The attribute C{self.root} is the tree's root L{Node}.
    """

    def __init__(self):
        self.root = None
        super(Tree, self).__init__()

    def __repr__(self):
        return repr(self.root)

    def __str__(self):
        # need to override networkx.DiGraph.__str__
        return ('Abstract syntax tree as graph with edges:\n' +
                str([(str(u), str(v))
                    for u, v, k in self.edges_iter(keys=True)]))

    @property
    def variables(self):
        """Return the set of variables in C{tree}.

        @rtype: C{set} of L{Var}
        """
        return {u for u in self if u.type == 'var'}

    @classmethod
    def from_recursive_ast(cls, u):
        tree = cls()
        tree.root = u
        tree._recurse(u)
        return tree

    def _recurse(self, u):
        if hasattr(u, 'value'):
            # necessary this terminal is the root
            self.add_node(u)
        elif hasattr(u, 'operator'):
            for i, v in enumerate(u.operands):
                self.add_edge(u, v, key=i)
                self._recurse(v)
        else:
            raise Exception('unknown node type: {u}'.format(u=u))
        return u

    def to_recursive_ast(self, u=None):
        if u is None:
            u = self.root
        w = copy.copy(u)
        if not self.succ.get(u):
            assert hasattr(u, 'value')
        else:
            w.operands = [self.to_recursive_ast(v)
                          for _, v, _ in sorted(
                              self.edges_iter(u, keys=True),
                              key=lambda x: x[2])]
            assert len(u.operands) == len(w.operands)
        return w

    def add_subtree(self, leaf, tree):
        """Add the C{tree} at node C{nd}.

        @type leaf: L{FOL.Node}

        @param tree: to be added, w/o copying AST nodes.
        @type tree: L{Tree}
        """
        assert not self.succ.get(leaf)
        for u, v, k in tree.edges_iter(keys=True):
            self.add_edge(u, v, key=k)
        # replace old leaf with subtree root
        ine = self.in_edges(leaf, keys=True)
        if ine:
            assert len(ine) == 1
            ((parent, _, k), ) = ine
            self.add_edge(parent, tree.root, key=k)
        else:
            self.root = tree.root
        self.remove_node(leaf)

    def to_pydot(self, detailed=False):
        """Create GraphViz dot string from given AST.

        @type ast: L{ASTNode}
        @rtype: str
        """
        g = ast_to_labeled_graph(self, detailed)
        return nx.drawing.nx_pydot.to_pydot(g)

    def write(self, filename, detailed=False):
        """Layout AST and save result in PDF file."""
        fname, fext = os.path.splitext(filename)
        fext = fext[1:]  # drop .
        p = self.to_pydot(detailed)
        p.set_ordering('out')
        p.write(filename, format=fext)


def ast_to_labeled_graph(tree, detailed):
    """Convert AST to C{NetworkX.DiGraph} for graphics.

    @param ast: Abstract syntax tree

    @rtype: C{networkx.DiGraph}
    """
    g = nx.DiGraph()
    for u in tree:
        if hasattr(u, 'operator'):
            label = u.operator
        elif hasattr(u, 'value'):
            label = u.value
        else:
            raise TypeError(
                'AST node must be Operator or Terminal, '
                'got instead: {u}'.format(u=u) +
                ', of type: {t}'.format(t=type(u)))
        # show both repr and AST node class in each vertex
        if detailed:
            label += '\n' + str(type(u).__name__)
        g.add_node(id(u), label=label)
    for u, v, k in tree.edges_iter(keys=True):
        g.add_edge(id(u), id(v), label=k)
    return g


def check_for_undefined_identifiers(tree, domains):
    """Check that types in C{tree} are incompatible with C{domains}.

    Raise a C{ValueError} if C{tree} either:

      - contains a variable missing from C{domains}
      - binary operator between variable and
        invalid value for that variable.

    @type tree: L{Tree}

    @param domains: variable definitions:

        C{{'varname': domain}}

        See L{GRSpec} for more details of available domain types.
    @type domains: C{dict}
    """
    for u in tree:
        if u.type == 'var' and u.value not in domains:
            var = u.value
            raise ValueError(
                ('Undefined variable "{var}" missing from '
                 'symbol table:\n\t{doms}\n'
                 'in subformula:\n\t{f}').format(
                     var=var, f=tree.to_recursive_ast(), doms=domains))

        if u.type not in {'str', 'num'}:
            continue

        # is a Const or Num
        var, c = pair_node_to_var(tree, u)

        if c.type == 'str':
            dom = domains[var]

            if not isinstance(dom, list):
                raise Exception(
                    ('String constant "{c}" assigned to non-string '
                     'variable "{var}" with domain:\n\t{dom}').format(
                         var=var, c=c, dom=dom))

            if c.value not in domains[var.value]:
                raise ValueError(
                    ('String constant "{c}" is not in the domain '
                     'of variable "{var}"').format(var=var, c=c))

        if c.type == 'num':
            dom = domains[var]

            if not isinstance(dom, tuple):
                raise Exception(
                    ('Number: {c}, assigned to non-integer ' + str(c) +
                     'variable "{var}" with domain:\n\t{dom}').format(
                         var=var, c=c, dom=dom))

            if not dom[0] <= c.value <= dom[1]:
                raise Exception(
                    ('Integer variable "{var}", is assigned the '
                     'value: {c}, that is out of its domain:'
                     '{dom[0]} ... {dom[1]}').format(
                         var=var, c=c, dom=dom))


def sub_values(tree, var_values):
    """Substitute given values for variables.

    @param tree: AST

    @type var_values: C{dict}

    @return: AST with L{Var} nodes replaces by
        L{Num}, L{Const}, or L{Bool}
    """
    old2new = dict()
    for u in tree.nodes_iter():
        if u.type != 'var':
            continue
        val = var_values[u.value]
        # instantiate appropriate value type
        if isinstance(val, bool):
            v = nodes.Bool(val)
        elif isinstance(val, int):
            v = nodes.Num(val)
        elif isinstance(val, str):
            v = nodes.Str(val)
        old2new[u] = v
    # replace variable by value
    nx.relabel_nodes(tree, old2new, copy=False)


def sub_constants(tree, var_str2int):
    """Replace string constants by integers.

    To be used for converting arbitrary finite domains
    to integer domains prior to calling gr1c.

    @param const2int: {'varname':['const_val0', ...], ...}
    @type const2int: C{dict} of C{list}
    """
    # logger.info('substitute ints for constants in:\n\t' + str(self))
    old2new = dict()
    for u in tree.nodes_iter():
        if u.type != 'str':
            continue
        var, op = pair_node_to_var(tree, u)
        # now: c, is the operator and: v, the variable
        str2int = var_str2int[str(var)]
        x = str2int.index(u.value)
        num = nodes.Num(str(x))
        # replace Const with Num
        old2new[u] = num
    nx.relabel_nodes(tree, old2new, copy=False)
    # logger.info('result after substitution:\n\t' + str(self) + '\n')


def sub_bool_with_subtree(tree, bool2subtree):
    """Replace selected Boolean variables with given AST.

    @type tree: L{LTL_AST}

    @param bool2form: map from each Boolean variable to some
        equivalent formula. A subset of Boolean varibles may be used.

        Note that the types of variables in C{tree}
        are defined by C{bool2form}.
    @type bool2form: C{dict} from C{str} to L{Tree}
    """
    for u in tree.nodes():
        if u.type == 'var' and u.value in bool2subtree:
            # tree.write(str(id(tree)) + '_before.png')
            tree.add_subtree(u, bool2subtree[u.value])
            # tree.write(str(id(tree)) + '_after.png')


def pair_node_to_var(tree, c):
    """Find variable under L{Binary} operator above given node.

    First move up from C{nd}, stop at first L{Binary} node.
    Then move down, until first C{Var}.
    This assumes that only L{Unary} operators appear between a
    L{Binary} and its variable and constant operands.

    May be extended in the future, depending on what the
    tools support and is thus needed here.

    @type tree: L{LTL_AST}

    @type L{nd}: L{Const} or L{Num}

    @return: variable, constant
    @rtype: C{(L{Var}, L{Const})}
    """
    # find parent Binary operator
    while True:
        old = c
        c = next(iter(tree.predecessors_iter(c)))
        if c.type == 'operator':
            if len(c.operands) == 2:
                break
    p, q = tree.successors_iter(c)
    v = p if q == old else q
    # go down until terminal found
    # assuming correct syntax for gr1c
    while True:
        if not tree.succ.get(v):
            break
        v = next(iter(tree.successors_iter(v)))
    # now: b, is the operator and: v, the variable
    return v, c


def infer_constants(formula, variables):
    """Enclose all non-variable names in quotes.

    @param formula: well-formed LTL formula
    @type formula: C{str} or L{LTL_AST}

    @param variables: domains of variables, or only their names.
        If the domains are given, then they are checked
        for ambiguities as for example a variable name
        duplicated as a possible value in the domain of
        a string variable (the same or another).

        If the names are given only, then a warning is raised,
        because ambiguities cannot be checked in that case,
        since they depend on what domains will be used.
    @type variables: C{dict} as accepted by L{GRSpec} or
        container of C{str}

    @return: C{formula} with all string literals not in C{variables}
        enclosed in double quotes
    @rtype: C{str}
    """
    if isinstance(variables, dict):
        for var in variables:
            other_vars = dict(variables)
            other_vars.pop(var)
            _check_var_conflicts({var}, other_vars)
    else:
        logger.error('infer constants does not know the variable domains.')
        warnings.warn(
            'infer_constants can give an incorrect result '
            'depending on the variable domains.\n'
            'If you give the variable domain definitions as dict, '
            'then infer_constants will check for ambiguities.')
    tree = parser.parse(formula)
    old2new = dict()
    for u in tree:
        if u.type != 'var':
            continue
        if str(u) in variables:
            continue
        # Var (so NAME token) but not a variable
        # turn it into a string constant
        old2new[u] = nodes.Const(str(u))
    nx.relabel_nodes(tree, old2new, copy=False)
    return str(tree)


def _check_var_conflicts(s, variables):
    """Raise exception if set intersects existing variable name, or values.

    Values refers to arbitrary finite data types.

    @param s: set

    @param variables: definitions of variable types
    @type variables: C{dict}
    """
    # check conflicts with variable names
    vars_redefined = {x for x in s if x in variables}
    if vars_redefined:
        raise Exception('Variables redefined: {v}'.format(v=vars_redefined))
    # check conflicts with values of arbitrary finite data types
    for var, domain in variables.iteritems():
        # not arbitrary finite type ?
        if not isinstance(domain, list):
            continue
        # var has arbitrary finite type
        conflicting_values = {x for x in s if x in domain}
        if conflicting_values:
            raise Exception(
                'Values redefined: {v}'.format(v=conflicting_values))


def check_var_name_conflict(f, varname):
    t = parser.parse(f)
    g = Tree.from_recursive_ast(t)
    v = {x.value for x in g.variables}
    if varname in v:
        raise ValueError('var name "{v}" already used'.format(v=varname))
    return v


def collect_primed_vars(t):
    """Return `set` of variable identifiers in the context of a next operator.

    @type t: recursive AST
    """
    g = Tree.from_recursive_ast(t)
    # (node, context)
    Q = [(t, False)]
    primed = set()
    while Q:
        u, c = Q.pop()
        if u.type == 'var' and c:
            primed.add(u.value)
        try:
            c = (u.operator == 'X') or c
        except AttributeError:
            pass
        Q.extend((v, c) for v in g.successors_iter(u))
    return primed


# defunct until further notice
def _flatten(tree, u, to_lang, **kw):
    """Recursively flatten C{tree}.

    @rtype: C{str}
    """
    s = tree.succ[u]
    if not s:
        return to_lang(u, **kw)
    elif len(s) == 2:
        l, r = s
        if 1 in s[l]:
            l, r = r, l
        l = _flatten(tree, l, to_lang, **kw)
        r = _flatten(tree, r, to_lang, **kw)
        return to_lang(u, l, r, **kw)
    else:
        (c,) = s
        if u.op == 'X':
            return to_lang(u, _flatten(tree, c, to_lang,
                           prime=True, **kw), **kw)
        else:
            return to_lang(u, _flatten(tree, c, to_lang, **kw), **kw)



"""
example to illustrate the combined use of
an environment and a system transition system.
"""
import logging
logging.basicConfig(filename='sys_and_env_ts.log',
                    level=logging.DEBUG, filemode='w')
logger = logging.getLogger(__name__)

from tulip import transys, spec, synth

# the system's spatial layout:
#     +----+----+----+
#     | X3 | X4 | X5 |
#     +----+----+----+
#     | X0 | X1 | X2 |
#     +----+----+----+

sys = transys.FTS()
sys.states.add_from(['X0', 'X1', 'X2', 'X3', 'X4', 'X5'])
sys.states.initial.add_from(['X0', 'X1'])

sys.transitions.add_from(
    [('X0', x) for x in {'X1', 'X3'}] +
    [('X1', x) for x in {'X0', 'X4', 'X2'}] +
    [('X2', x) for x in {'X1', 'X5'}] +
    [('X3', x) for x in {'X0', 'X4'}] +
    [('X4', x) for x in {'X3', 'X1', 'X5'}] +
    # compared to home_parking, this car can stay in lot next
    [('X5', x) for x in {'X4', 'X2', 'X5'}]
)

sys.atomic_propositions.add_from({'home', 'lot'})
sys.states.add('X0', ap={'home'})
sys.states.add('X5', ap={'lot'})

"""Park as an env AP
"""
env0 = transys.FTS()
env0.owner = 'env'
env0.states.add_from({'e0', 'e1'})
env0.states.initial.add_from({'e0', 'e1'})

env0.atomic_propositions.add('park')
env0.states.add('e0', ap={'park'})

env0.transitions.add_from([
    ('e0', 'e0'), ('e0', 'e1'),
    ('e1', 'e0'), ('e1', 'e1')
])
logger.info(env0)

# barely realizable: assumption necessary
env_prog = '!park'

sys_vars = {'mem'}
sys_init = {'mem'}
sys_prog = {'home'}

# one additional requirement: if in lot,
# then stay there until park signal is turned off
sys_safe = {'(X(mem) <-> lot) || (mem && !park)',
            '((lot && park) -> X(lot))'}
sys_prog |= {'mem'}

specs = spec.GRSpec(sys_vars=sys_vars, sys_init=sys_init,
                    sys_safety=sys_safe,
                    env_prog=env_prog, sys_prog=sys_prog)
specs.moore = False
specs.qinit = '\A \E'
ctrl = synth.synthesize('omega', specs, sys=sys, env=env0)
ctrl.save('sys_and_env_ts0.pdf')
logger.info(ctrl)

"""Park as an env action
"""
env1 = transys.FTS()
env1.owner = 'env'
env1.states.add('e0')
env1.states.initial.add('e0')

env1.env_actions.add_from({'park', 'none'})

env1.transitions.add('e0', 'e0', env_actions='park')
env1.transitions.add('e0', 'e0', env_actions='none')
logger.info(env1)

env_prog = ['! (env_actions = "park")']
sys_safe = {'(X(mem) <-> lot) || (mem && ! (env_actions = "park"))',
            '((lot && (env_actions = "park")) -> X(lot))'}

specs = spec.GRSpec(sys_vars=sys_vars, sys_init=sys_init,
                    sys_safety=sys_safe,
                    env_prog=env_prog, sys_prog=sys_prog)
specs.moore = False
specs.qinit = '\A \E'
ctrl = synth.synthesize('omega', specs, sys=sys, env=env1)
ctrl.save('sys_and_env_ts1.pdf')
env1.save('env1.pdf')
logger.info(ctrl)



#!/usr/bin/env python
"""Tests for the interface with slugs."""
import logging
logger = logging.getLogger(__name__)
from tulip.interfaces import slugs
import jtlvint_test


def bitfields_to_ints_test():
    t = {'a': (0, 30)}

    # test int values
    bits = {'a@0.0.30': 0, 'a@1': 1, 'a@2': 1,
            'a@3': 0, 'a@4': 1, 'a@5': 0}
    n = slugs._bitfields_to_ints(bits, t)
    logger.debug(n)
    assert n == {'a': 22}

    # test str values
    bits = {'a@0.0.30': '0', 'a@1': '1', 'a@2': '1',
            'a@3': '0', 'a@4': '1', 'a@5': '0'}
    n = slugs._bitfields_to_ints(bits, t)
    logger.debug(n)
    assert n == {'a': 22}

    # range
    for n in xrange(30):
        bits = list(bin(n).lstrip('0b').zfill(6))
        bits.reverse()  # little-endian
        d = {'a@{i}'.format(i=i): v for i, v in enumerate(bits)}
        d['a@0.0.30'] = d.pop('a@0')
        t = {'a': (0, 30)}
        print(d)
        m = slugs._bitfields_to_ints(d, t)
        logger.debug((n, m))
        assert m == {'a': n}


class basic_test(jtlvint_test.basic_test):
    def setUp(self):
        super(basic_test, self).setUp()
        self.check_realizable = lambda x: slugs.synthesize(x) is not None
        self.synthesize = slugs.synthesize



#!/usr/bin/env python
"""Generate bibliography.rst

The input file is of the following form. Blank lines are ignored. New
lines are collapsed to a single space. Besides newlines, text for each
entry is copied without filtering, and thus reST syntax can be
used. Note that the key line must begin with '['.

[M55]
G. H. Mealy. `A Method for Synthesizing Sequential Circuits
<http://dx.doi.org/10.1002/j.1538-7305.1955.tb03788.x>`_. *Bell System
Technical Journal (BSTJ)*, Vol.34, No.5, pp. 1045 -- 1079, September,
1955.
"""
from __future__ import print_function
import sys
import io


def print_entry(out_f, bkey, entry_text):
    nl = '\n'
    idt = 4*' '
    if bkey is not None:
        bkey_canon = bkey.lower()
        out_f.write(':raw-html:`<a href="#'+bkey_canon+'" id="'+bkey_canon+
                    '">['+bkey+']</a>`'+nl+idt+'\\'+entry_text+2*nl)

if len(sys.argv) < 2:
    print('Usage: genbib.py FILE')
    sys.exit(1)

out_f = io.open('bibliography.rst', 'w')

with io.open(sys.argv[1], 'r') as f:
    bkey = None
    entry_text = None
    out_f.write(u'''Bibliography
============

.. role:: raw-html(raw)
   :format: html

''')
    for line in f:
        line = line.strip()
        if len(line) == 0:
            continue

        if line[0] == '[' and line:
            print_entry(out_f, bkey, entry_text)
            closing_sym = line.index(']')
            bkey = line[1:closing_sym]
            entry_text = u''
        elif bkey is None:
            ValueError('Entry text found without preceding key.')
        else:
            entry_text += line+' '

print_entry(out_f, bkey, entry_text)



#!/usr/bin/env python
"""Simulation example with continuous dynamics."""
import logging
import random

import numpy as np
import polytope as pc
from polytope import box2poly
try:
    from matplotlib import pyplot as plt
except ImportError:
    plt = None

from tulip import hybrid, spec, synth
from tulip.abstract import prop2part, discretize
from tulip.abstract.plot import plot_partition
from tulip.abstract import find_controller


logging.basicConfig(level='WARNING')


show = False
# Problem parameters
input_bound = 10.0
uncertainty = 0.01
# Continuous state space
cont_state_space = box2poly([[0., 3.], [0., 2.]])
# Continuous dynamics
A = np.array([[1.0, 0.], [0., 1.0]])
B = np.array([[0.1, 0.], [0., 0.1]])
E = np.array([[1, 0], [0, 1]])
# Available control, possible disturbances
U = input_bound * np.array([[-1., 1.], [-1., 1.]])
W = uncertainty * np.array([[-1., 1.], [-1., 1.]])
# Convert to polyhedral representation
U = box2poly(U)
W = box2poly(W)
# Construct the LTI system describing the dynamics
sys_dyn = hybrid.LtiSysDyn(A, B, E, None, U, W, cont_state_space)
# Define atomic propositions for relevant regions of state space
cont_props = {}
cont_props['home'] = box2poly([[0., 1.], [0., 1.]])
cont_props['lot'] = box2poly([[2., 3.], [1., 2.]])
# Compute the proposition preserving partition of
# the continuous state space
cont_partition = prop2part(cont_state_space, cont_props)
plot_partition(cont_partition) if show else None
# Given dynamics & proposition-preserving partition,
# find feasible transitions
disc_dynamics = discretize(
    cont_partition, sys_dyn, closed_loop=False,
    conservative=True,
    N=5, min_cell_volume=0.1, plotit=show)
# Visualize transitions in continuous domain (optional)
plot_partition(disc_dynamics.ppp, disc_dynamics.ts,
               disc_dynamics.ppp2ts) if show else None
#
# Specification
# Environment variables and assumptions
env_vars = {'park'}
env_init = set()
env_prog = '!park'
env_safe = set()
# System variables and requirements
sys_vars = {'X0reach'}
sys_init = {'X0reach'}
sys_prog = {'home'}  # []<>home
sys_safe = {'(X(X0reach) <-> lot) || (X0reach && !park)'}
sys_prog |= {'X0reach'}
# Create the specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
specs.qinit = '\E \A'
#
# Synthesize
ctrl = synth.synthesize('omega', specs,
                        sys=disc_dynamics.ts,
                        ignore_sys_init=True)
assert ctrl is not None, 'unrealizable'
# Generate a graphical representation of the controller for viewing
if not ctrl.save('continuous.png'):
    print(ctrl)
#
# Simulation
print('\n Simulation starts \n')
T = 100
# let us pick an environment signal
randParkSignal = [random.randint(0, 1) for b in range(1, T + 1)]
# initialization:
#     pick initial continuous state consistent with
#     initial controller state (discrete)
u, v, edge_data = ctrl.edges('Sinit', data=True)[1]
s0_part = edge_data['loc']
init_poly_v = pc.extreme(disc_dynamics.ppp[s0_part][0])
x_init = sum(init_poly_v) / init_poly_v.shape[0]
x = [x_init[0]]
y = [x_init[1]]
N = disc_dynamics.disc_params['N']
s0_part = find_controller.find_discrete_state(
    [x[0], y[0]], disc_dynamics.ppp)
ctrl = synth.determinize_machine_init(ctrl, {'loc': s0_part})
(s, dum) = ctrl.reaction('Sinit', {'park': randParkSignal[0]})
print(dum)
for i in range(0, T):
    (s, dum) = ctrl.reaction(s, {'park': randParkSignal[i]})
    u = find_controller.get_input(
        x0=np.array([x[i * N], y[i * N]]),
        ssys=sys_dyn,
        abstraction=disc_dynamics,
        start=s0_part,
        end=disc_dynamics.ppp2ts.index(dum['loc']),
        ord=1,
        mid_weight=5)
    for ind in range(N):
        s_now = np.dot(
            sys_dyn.A, [x[-1], y[-1]]
        ) + np.dot(sys_dyn.B, u[ind])
        x.append(s_now[0])
        y.append(s_now[1])
    s0_part = find_controller.find_discrete_state(
        [x[-1], y[-1]], disc_dynamics.ppp)
    s0_loc = disc_dynamics.ppp2ts[s0_part]
    print(s0_loc)
    print(dum['loc'])
    print(dum)
show_traj = True
if show_traj:
    assert plt, 'failed to import matplotlib'
    plt.plot(x)
    plt.plot(y)
    plt.show()



# Copyright (c) 2011-2015 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
"""AST subclasses to translate to each syntax of:

  - gr1c: https://tulip-control.github.io/gr1c/md_spc_format.html
  - JTLV
  - SMV: http://nusmv.fbk.eu/NuSMV/userman/v21/nusmv_3.html
  - SPIN: http://spinroot.com/spin/Man/ltl.html
          http://spinroot.com/spin/Man/operators.html
  - python (Boolean formulas only)
  - WRING: http://vlsi.colorado.edu/~rbloem/wring.html
        (see top of file: LTL.pm)
"""
import logging
logger = logging.getLogger(__name__)
# import pprint
import re
from tulip.spec import ast
import tulip.spec.form


def make_jtlv_nodes():
    opmap = {
        'False': 'FALSE', 'True': 'TRUE',
        '!': '!',
        '|': '|', '&': '&', '->': '->', '<->': '<->',
        'G': '[]', 'F': '<>', 'X': 'next',
        'U': 'U',
        '<': '<', '<=': '<=', '=': '=', '>=': '>=', '>': '>', '!=': '!='}
    nodes = ast.make_fol_nodes(opmap)

    class Str(nodes.Str):
        def flatten(self, **kw):
            return '({c})'.format(c=self)

    class Var(nodes.Var):
        def flatten(self, env_vars=None, sys_vars=None, **kw):
            v = self.value
            if v in env_vars:
                player = 'e'
            elif v in sys_vars:
                player = 's'
            else:
                raise ValueError('{v} neither env nor sys var'.format(v))
            return '({player}.{value})'.format(player=player, value=v)

    nodes.Str = Str
    nodes.Var = Var
    return nodes


def make_gr1c_nodes(opmap=None):
    if opmap is None:
        opmap = {
            'False': 'False', 'True': 'True',
            '!': '!',
            '|': '|', '&': '&', '->': '->', '<->': '<->',
            'G': '[]', 'F': '<>', 'X': '',
            '<': '<', '<=': '<=', '=': '=',
            '>=': '>=', '>': '>', '!=': '!='}
    nodes = ast.make_fol_nodes(opmap)

    class Var(nodes.Var):
        def flatten(self, prime=None, **kw):
            return '{v}{prime}'.format(
                v=self.value, prime="'" if prime else '')

    class Unary(nodes.Unary):
        def flatten(self, *arg, **kw):
            if self.operator == 'X':
                kw.update(prime=True)
                return self.operands[0].flatten(*arg, **kw)
            return super(Unary, self).flatten(*arg, **kw)

    nodes.Var = Var
    nodes.Unary = Unary
    return nodes


def make_slugs_nodes():
    """Simple translation, unisigned arithmetic only.

    For signed arithmetic use Promela instead.
    """
    opmap = {
        'False': 'FALSE', 'True': 'TRUE',
        '!': '!',
        '|': '|', '&': '&', '->': '->', '<->': '<->',
        'G': '[]', 'F': '<>', 'X': '',
        '<': '<', '<=': '<=', '=': '=', '>=': '>=', '>': '>', '!=': '!=',
        '+': '+', '-': '-'}  # linear arithmetic
    return make_gr1c_nodes(opmap)


def make_promela_nodes():
    opmap = dict(ast.OPMAP)
    opmap.update({'True': 'true', 'False': 'false',
                  'G': '[]', 'F': '<>', 'R': 'V', '=': '=='})
    return ast.make_fol_nodes(opmap)


def make_smv_nodes():
    opmap = {'X': 'X', 'G': 'G', 'F': 'F', 'U': 'U', 'R': 'V'}
    return ast.make_fol_nodes(opmap)

def make_wring_nodes():
    opmap = {'False': '0', 'True': '1',
             '!': '!',
             '|': '+', '&': '*', '->': '->', '<->': '<->', 'xor': '^',
             'G': 'G', 'F': 'F', 'X': 'X',
             'U': 'U', 'R': 'R', 'V': 'V'}
    nodes = ast.make_fol_nodes(opmap)

    class Var(nodes.Var):
        def flatten(self, *arg, **kw):
            if kw.has_key('env_vars') or kw.has_key('sys_vars'):
                env_vars = kw['env_vars']
                sys_vars = kw['sys_vars']
                if env_vars.has_key(self.value):
                    this_type = env_vars[self.value]
                elif sys_vars.has_key(self.value):
                    this_type = sys_vars[self.value]
                else:
                    raise TypeError(
                        '"{v}" is not defined as a variable in {t1} nor {t2}'.format(
                            v=self.value, t1=env_vars, t2=sys_vars))

                if this_type != 'boolean':
                    raise TypeError('"{v}" is not Boolean, but {type}'.format(
                        v=self.val, type=this_type))
            return '({var}=1)'.format(var=self.value)

    nodes.Var = Var
    return nodes

def make_python_nodes():
    opmap = {'True': 'True', 'False': 'False',
             '!': 'not', '&': 'and', '|': 'or',
             '^': '^', '=': '==', '!=': '!=',
             '<': '<', '<': '<',
             '>=': '>=', '<=': '<=', '>': '>',
             '+': '+', '-': '-'}
    nodes = ast.make_fol_nodes(opmap)

    class Imp(nodes.Binary):
        def flatten(self, *arg, **kw):
            return '((not ({l})) or {r})'.format(
                l=self.operands[0].flatten(),
                r=self.operands[1].flatten())

    class BiImp(nodes.Binary):
        def flatten(self, *arg, **kw):
            return '({l} == {r})'.format(
                l=self.operands[0].flatten(),
                r=self.operands[1].flatten())

    nodes.Imp = Imp
    nodes.BiImp = BiImp
    return nodes


lang2nodes = {
    'jtlv': make_jtlv_nodes(),
    'gr1c': make_gr1c_nodes(),
    'slugs': make_slugs_nodes(),
    'promela': make_promela_nodes(),
    'smv': make_smv_nodes(),
    'python': make_python_nodes(),
    'wring': make_wring_nodes()}


def _to_jtlv(d):
    """Return specification as list of two strings [assumption, guarantee].

    Format is that of JTLV.  Cf. L{interfaces.jtlv}.
    """
    logger.info('translate to jtlv...')
    f = _jtlv_str
    parts = [f(d['env_init'], 'valid initial env states', ''),
             f(d['env_safety'], 'safety assumption on environment', '[]'),
             f(d['env_prog'], 'justice assumption on environment', '[]<>')]
    assumption = ' & \n'.join(x for x in parts if x)

    parts = [f(d['sys_init'], 'valid initial system states', ''),
             f(d['sys_safety'], 'safety requirement on system', '[]'),
             f(d['sys_prog'], 'progress requirement on system', '[]<>')]
    guarantee = ' & \n'.join(x for x in parts if x)
    return (assumption, guarantee)


def _jtlv_str(m, comment, prefix='[]<>'):
    # no clauses ?
    if not m:
        return ''
    w = list()
    for x in m:
        logger.debug('translate clause: ' + str(x))
        if not x:
            continue
        # collapse any whitespace between any
        # "next" operator that precedes parenthesis
        if prefix == '[]':
            c = re.sub(r'next\s*\(', 'next(', x)
        else:
            c = x
        w.append('\t{prefix}({formula})'.format(prefix=prefix, formula=c))
    return '-- {comment}\n{formula}'.format(
        comment=comment, formula=' & \n'.join(w))


def _to_gr1c(d):
    """Dump to gr1c specification string.

    Cf. L{interfaces.gr1c}.
    """
    def _to_gr1c_print_vars(vardict):
        output = ''
        for var, dom in vardict.iteritems():
            if dom == 'boolean':
                output += ' ' + var
            elif isinstance(dom, tuple) and len(dom) == 2:
                output += ' %s [%d, %d]' % (var, dom[0], dom[1])
            elif isinstance(dom, list) and len(dom) > 0:
                int_dom = convert_domain(dom)
                output += ' %s [%d, %d]' % (var, int_dom[0], int_dom[1])
            else:
                raise ValueError(
                    'Domain "{dom}" not supported by gr1c.'.format(dom=dom))
        return output

    logger.info('translate to gr1c...')
    output = (
        'ENV:' + _to_gr1c_print_vars(d['env_vars']) + ';\n' +
        'SYS:' + _to_gr1c_print_vars(d['sys_vars']) + ';\n' +

        _gr1c_str(d['env_init'], 'ENVINIT', '') +
        _gr1c_str(d['env_safety'], 'ENVTRANS', '[]') +
        _gr1c_str(d['env_prog'], 'ENVGOAL', '[]<>') + '\n' +

        _gr1c_str(d['sys_init'], 'SYSINIT', '') +
        _gr1c_str(d['sys_safety'], 'SYSTRANS', '[]') +
        _gr1c_str(d['sys_prog'], 'SYSGOAL', '[]<>')
    )
    return output

def _to_wring(d):
    """Dump to LTL formula in Wring syntax

    Assume that d is a dictionary describing a GR(1) formula in the
    manner of tulip.spec.form.GRSpec; e.g., it should have a key named
    'env_init'. Compare with _to_gr1c().
    """
    assumption = ''
    if d['env_init']:
        assumption += ' * '.join(['('+f+')' for f in d['env_init']])
    if d['env_safety']:
        if len(assumption) > 0:
            assumption += ' * '
        assumption += ' * '.join(['G('+f+')' for f in d['env_safety']])
    if d['env_prog']:
        if len(assumption) > 0:
            assumption += ' * '
        assumption += ' * '.join(['G(F('+f+'))' for f in d['env_prog']])

    guarantee = ''
    if d['sys_init']:
        guarantee += ' * '.join(['('+f+')' for f in d['sys_init']])
    if d['sys_safety']:
        if len(guarantee) > 0:
            guarantee += ' * '
        guarantee += ' * '.join(['G('+f+')' for f in d['sys_safety']])
    if d['sys_prog']:
        if len(guarantee) > 0:
            guarantee += ' * '
        guarantee += ' * '.join(['G(F('+f+'))' for f in d['sys_prog']])

    # Put the parts together, simplifying in special cases
    if guarantee:
        if assumption:
            return '((' + assumption + ') -> (' + guarantee + '))'
        else:
            return '(' + guarantee + ')'
    else:
        return 'G(1)'

# currently also used in interfaces.jtlv
# eliminate it from there
def convert_domain(dom):
    """Return equivalent integer domain if C{dom} contais strings.

    @type dom: C{list} of C{str}
    @rtype: C{'boolean'} or C{(min_int, max_int)}
    """
    # not a string variable ?
    if not isinstance(dom, list):
        return dom
    return (0, len(dom) - 1)


def _gr1c_str(s, name='SYSGOAL', prefix='[]<>'):
    if not s:
        return '{name}:;\n'.format(name=name)
    f = '\n& '.join([
        prefix + '({u})'.format(u=x) for x in s])
    return '{name}: {f};\n'.format(name=name, f=f)


def _to_slugs(d):
    """Return structured slugs spec.
    """
    f = _slugs_str
    return (
        _format_slugs_vars(d['env_vars'], 'INPUT') +
        _format_slugs_vars(d['sys_vars'], 'OUTPUT') +

        f(d['env_safety'], 'ENV_TRANS') +
        f(d['env_prog'], 'ENV_LIVENESS') +
        f(d['env_init'], 'ENV_INIT', sep='&') +

        f(d['sys_safety'], 'SYS_TRANS') +
        f(d['sys_prog'], 'SYS_LIVENESS') +
        f(d['sys_init'], 'SYS_INIT', sep='&')
    )


def _slugs_str(r, name, sep='\n'):
    if not r:
        return '[{name}]\n'.format(name=name)
    sep = ' {sep} '.format(sep=sep)
    f = sep.join(x for x in r if x)
    return '[{name}]\n{f}\n\n'.format(name=name, f=f)


def _format_slugs_vars(vardict, name):
    a = []
    for var, dom in vardict.iteritems():
        if dom == 'boolean':
            a.append(var)
        elif isinstance(dom, tuple) and len(dom) == 2:
            a.append('{var}: {min}...{max}'.format(
                var=var, min=dom[0], max=dom[1])
            )
        else:
            raise ValueError('unknown domain type: {dom}'.format(dom=dom))
    return '[{name}]\n{vars}\n\n'.format(name=name, vars='\n'.join(a))


to_lang = {'jtlv': _to_jtlv, 'gr1c': _to_gr1c, 'slugs': _to_slugs,
           'wring': _to_wring}


def translate(spec, lang):
    """Return str or tuple in tool format.

    Consult the respective documentation in L{tulip.interfaces}
    concerning formats and links to further reading.

    @type spec: L{GRSpec}
    @type lang: 'gr1c', 'slugs', 'jtlv', or 'wring'

    @return: spec formatted for input to tool; the type of the return
    value depends on the tool:

        - C{str} if gr1c or slugs
        - (assumption, guarantee), where each element of the tuple is C{str}
    """
    if not isinstance(spec, tulip.spec.form.GRSpec):
        raise TypeError('translate requires first argument (spec) to be of type GRSpec')
    spec.check_syntax()
    spec.str_to_int()
    # pprint.pprint(spec._bool_int)
    d = {p: [translate_ast(spec.ast(spec._bool_int[x]), lang).flatten(
             env_vars=spec.env_vars, sys_vars=spec.sys_vars)
         for x in getattr(spec, p)] for p in spec._parts}
    # pprint.pprint(d)
    d['env_vars'] = spec.env_vars
    d['sys_vars'] = spec.sys_vars
    return to_lang[lang](d)


def translate_ast(tree, lang):
    """Return AST of formula C{tree}.

    @type tree: L{Nodes.Node}
    @type lang: 'gr1c' or 'slugs' or 'jtlv' or
      'promela' or 'smv' or 'python' or 'wring'

    @return: tree using AST nodes of C{lang}
    @rtype: L{FOL.Node}
    """
    if lang == 'python':
        return _ast_to_python(tree, lang2nodes[lang])
    else:
        return _ast_to_lang(tree, lang2nodes[lang])


def _ast_to_lang(u, nodes):
    cls = getattr(nodes, type(u).__name__)
    if hasattr(u, 'value'):
        return cls(u.value)
    elif hasattr(u, 'operator'):
        xyz = [_ast_to_lang(x, nodes) for x in u.operands]
        return cls(u.operator, *xyz)
    else:
        raise TypeError('Unknown node type "{t}"'.format(
            t=type(u).__name__))


def _ast_to_python(u, nodes):
    cls = getattr(nodes, type(u).__name__)
    if hasattr(u, 'value'):
        return cls(u.value)
    elif not hasattr(u, 'operands'):
        raise TypeError(
            'AST node: {u}'.format(u=type(u).__name__) +
            ', is neither terminal nor operator.')
    elif len(u.operands) == 1:
        assert u.operator == '!'
        return cls(u.operator, _ast_to_python(u.operands[0], nodes))
    elif len(u.operands) == 2:
        assert u.operator in {'&', '|', '^', '->', '<->',
                              '>', '>=', '=', '!=', '<=', '<',
                              '+', '-', '*', '/'}
        if u.operator == '->':
            cls = nodes.Imp
        elif u.operator == '<->':
            cls = nodes.BiImp
        return cls(u.operator,
                   _ast_to_python(u.operands[0], nodes),
                   _ast_to_python(u.operands[1], nodes))
    else:
        raise ValueError(
            'Operator: {u}, is neither unary nor binary.'.format(u=u))
